<?php

namespace Teknisa\CADFIN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\CADFIN\Factories\Ger40100CategoriasDreGerencialFactory;

/**
 * Class Ger40100CategoriasDreGerencialController
 * @package Teknisa\CADFIN\Services
 */
class Ger40100CategoriasDreGerencialService {

    protected $entityManager;
    protected $factory;
    
    /**
     * @param EntityManager $entityManager
     * @param Ger40100CategoriasDreGerencialFactory $factory
     */
    public function __construct(EntityManager $entityManager, Ger40100CategoriasDreGerencialFactory $factory)
    {
        $this->entityManager = $entityManager;
        $this->factory       = $factory;
    }

    /**
     * Atualiza Classificação Financeira - Categoria DRE
     * @param $nrorg
     * @param $cditemestru
     * @param $cdclassfina
     * @throws \Exception
     */
    public function atualizaClassFinancCategoriaDRE($nrorg, $cditemestru, $cdclassfina){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            
            // Busca Classificação Financeira
            /** @var Classfinanc $classfinanc */
            $classfinanc = $this->factory->buscaClassfinanc($nrorg, $cdclassfina);
            
            // Atualiza Classificação Financeira - Categoria DRE
            /** @var Classfinanc $classfinanc */
            $classfinanc = $this->factory->atualizaClassFinancCategoriaDRE($classfinanc, $cditemestru);
                
            $this->entityManager->persist($classfinanc);
            
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    

    /**
     * Remove Categoria DRE
     * @param $nrorg
     * @param $cditemestru
     * @throws \Exception
     */
    public function removeCategoriaDRE($nrorg, $cditemestru){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            
            // Busca Categoria DRE
            /** @var Dregerestru $dregerestru */
            $dregerestru = $this->factory->buscaDregerestru($nrorg, $cditemestru);
            
            // Busca Classificações Financeiras - Categoria DRE
            /** @var Classfinanc $classfinanc */
            $classfinancs = $this->factory->buscaClassfinancs($nrorg, $cditemestru);
            
            foreach($classfinancs as $classfinanc){
                // Atualiza Classificação Financeira - Categoria DRE
                /** @var Classfinanc $classfinanc */
                $classfinanc = $this->factory->atualizaClassFinancCategoriaDRE($classfinanc, '');
                
                $this->entityManager->persist($classfinanc);
                $this->entityManager->flush();
            }
            
            // Remove Categoria DRE
            $this->entityManager->remove($dregerestru);
            
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

}